/**
 * Copyright 1991-2006 General Electric Company.
 * All rights reserved.
 *
 * File Name: GEtimespec.h
 *
 */

#ifndef GEtimespec_INCL
#define GEtimespec_INCL

/**************************************************
 *                                                *
 * Windows                                        *
 *                                                *
 **************************************************/
#if defined(_WINDOWS)

#include "GEtypes.h"

struct __GEtimespec
{
    s32 tv_sec;
    s32 tv_nsec;
};

typedef struct __GEtimespec GEtimespec;

#define GETIMESPEC_TO_TIMESPEC(geT,t) \
    { \
    t.tv_sec = geT.tv_sec; \
    t.tv_nsec = geT.tv_nsec; \
    }

#define TIMESPEC_TO_GETIMESPEC(t,geT) \
    { \
    geT.tv_sec = (s32)t.tv_sec; \
    geT.tv_nsec = t.tv_nsec; \
    }
/***************************************************
 *                                                 *
 * 64-Bit Linux                                    *
 *                                                 *
 ***************************************************/
#elif defined(__linux__) && defined(__x86_64__)

#include <time.h>
#include "GEtypes.h"

/**
 * 64bit Linux OS version of timespec uses 64bit values. 
 * In order to be compatible with 32bit processes, 
 * GEtimespec will mimic the 32bit GNU version to timespec.
 */

struct __GEtimespec
{
    s32 tv_sec;
    s32 tv_nsec;
};

typedef struct __GEtimespec GEtimespec;

#define GETIMESPEC_TO_TIMESPEC(geT,t) \
    { \
    t.tv_sec = geT.tv_sec; \
    t.tv_nsec = geT.tv_nsec; \
    }
#define TIMESPEC_TO_GETIMESPEC(t,geT) \
    { \
    geT.tv_sec = 0x7fffffff; \
    geT.tv_nsec = 0x7fffffff; \
    if( (time_t)t.tv_sec <= 0x000000007fffffffLL ) \
    { \
        geT.tv_sec = t.tv_sec; \
    } \
    geT.tv_nsec = t.tv_nsec; \
    }
#else
/***************************************************
 *                                                 *
 * All other systems, assumes 32-bit operation.    *
 *                                                 *
 ***************************************************/

#include <time.h>

typedef struct timespec GEtimespec;

#define GETIMESPEC_TO_TIMESPEC(geT,t) \
    t =  (struct timespec)geT

#define TIMESPEC_TO_GETIMESPEC(t,geT) \
    geT =  (GEtimespec)t

#endif /* defined(_WINDOWS) */

#endif /* GEtimespec_INCL */
