/**
 * Copyright 1991-2005 General Electric Company.
 * All rights reserved.
 *
 * File Name: GEtypes.h
 *
 */


#ifndef GEtypes_INCL
#define GEtypes_INCL

/**************************************************
 *                                                *
 * OS       :   Windows                           *
 *                                                *
 **************************************************/
#if defined(_WINDOWS)


/**************************************************
 *                                                *
 * Compiler :   MS VC++                           *
 * Version  :   tested on 7.0                     *
 *                                                *
 **************************************************/
#if defined(_MSC_VER)

#include <stddef.h>

/* VC++ provides built in types for different sizes of integers */
typedef __int8             s8;
typedef unsigned __int8    n8;
typedef __int16            s16;
typedef unsigned __int16   n16;
typedef __int32            s32;
typedef unsigned __int32   n32;
typedef __int64            s64;
typedef unsigned __int64   n64;

typedef float              f32;
typedef double             f64;


/* define missing ANSI types: */
typedef s8                     int_least8_t;
typedef n8                     uint_least8_t;
typedef s16                    int_least16_t;
typedef n16                    uint_least16_t;
typedef s32                    int_least32_t;
typedef n32                    uint_least32_t;
typedef s64                    int_least64_t;
typedef n64                    uint_least64_t;

typedef int                    int_fast8_t;
typedef unsigned int           uint_fast8_t;
typedef int                    int_fast16_t;
typedef unsigned int           uint_fast16_t;
typedef int                    int_fast32_t;
typedef unsigned int           uint_fast32_t;
typedef s64                    int_fast64_t;
typedef n64                    uint_fast64_t;

typedef s64                    intmax_t;
typedef n64                    uintmax_t;


/**
 * Architecture specific stuff:
 */
#if defined(WIN64)

#define GE_POINTER_SIZE 8

typedef s64                    ssize_t;

#else

#define GE_POINTER_SIZE 4

typedef s32                    ssize_t;

#endif

#endif /* _MSC_VER */

/***************************************************
 *                                                 *
 * OS       :   Linux (RedHat)                     *
 *                                                 *
 ***************************************************/
#elif defined(__linux__)

/***************************************************
 *                                                 *
 * Compiler :   GNU                                *
 * Version  :   tested on 3.3.3                    *
 *                                                 *
 ***************************************************/

#include <stddef.h>
#include <stdint.h>

/**
 * 64 bit section
 */
#if defined(__x86_64__)


typedef int8_t             s8;
typedef uint8_t            n8;
typedef int16_t            s16;
typedef uint16_t           n16;
typedef int32_t            s32;
typedef uint32_t           n32;
typedef int64_t            s64;
typedef uint64_t           n64;

typedef float              f32;
typedef double             f64;

#define GE_POINTER_SIZE 8

/**
 * 32 bit section
 */
#else

/*
 * it is assumed that a GNU compiler is used  use these 
 * declarations for the GNU gcc compiler (as defined in 
 * sys/types.h)
 */
#include <sys/types.h>

typedef char               s8;
typedef unsigned char      n8;
typedef int16_t            s16;
typedef u_int16_t          n16;
typedef long               s32;
typedef unsigned long      n32;
/* use long long int for Objective C compiler */
#ifdef __OBJC__
typedef long long          s64;
#else
typedef int64_t            s64;
#endif
typedef uint64_t           n64;

typedef float              f32;
typedef double             f64;

#define GE_POINTER_SIZE 4


#endif /* __x86_64__ */

/****************************************************************
 *                                                              *
 * OS       :   VxWorks 5.5                                     *
 * Compiler :   GNU                                             *
 * Version  :   2.96 (2.96+ MW/LM) 19990621 AltiVec VxWorks 5.5 *
 *                                                              *
 ****************************************************************/
#elif defined(VXWORKS) /* defined(VXWORKS) */

#ifdef __GNUC__
#include <sys/types.h>
#endif

typedef char                   s8;
typedef unsigned char          n8;
typedef short                  s16;
typedef unsigned short         n16;
typedef long                   s32;
typedef unsigned long          n32;
typedef long long int          s64;
typedef unsigned long long int n64;

typedef float                  f32;
typedef double                 f64;

#define GE_POINTER_SIZE 4


/* define missing ANSI types: */
typedef unsigned int           uintptr_t;
typedef int                    intptr_t;

typedef s8                     int_least8_t;
typedef n8                     uint_least8_t;
typedef s16                    int_least16_t;
typedef n16                    uint_least16_t;
typedef s32                    int_least32_t;
typedef n32                    uint_least32_t;
typedef s64                    int_least64_t;
typedef n64                    uint_least64_t;

typedef int                    int_fast8_t;
typedef unsigned int           uint_fast8_t;
typedef int                    int_fast16_t;
typedef unsigned int           uint_fast16_t;
typedef int                    int_fast32_t;
typedef unsigned int           uint_fast32_t;
typedef s64                    int_fast64_t;
typedef n64                    uint_fast64_t;
typedef s64                    intmax_t;
typedef n64                    uintmax_t;


/****************************************************************
 *                                                              *
 * OS       :                                                   *
 * Compiler :   C2000                                           *
 * Version  :                                                   *
 *                                                              *
 ****************************************************************/
#elif defined(_TMS320C2XX) /* C2000 Compiler */
typedef char                   s8;
typedef unsigned char          n8;
typedef short                  s16;
typedef unsigned short         n16;
typedef long                   s32;
typedef unsigned long          n32;

#ifdef ENABLE_GETYPE_64BIT_DEFINE

#error 64bit GEtypes not defined for this platform.

#endif /* ENABLE_GETYPE_64BIT_DEFINE */

typedef float                  f32;
typedef double                 f64;



/****************************************************************
 *                                                              *
 * OS       :                                                   *
 * Compiler :   C6X                                             *
 * Version  :                                                   *
 *                                                              *
 ****************************************************************/
#elif defined(_TMS320C6X)

typedef char                   s8;
typedef unsigned char          n8;
typedef short                  s16;
typedef unsigned short         n16;
typedef int                    s32;
typedef unsigned int           n32;

#ifdef ENABLE_GETYPE_64BIT_DEFINE

#error 64bit GEtypes not defined for this platform.

#endif /* ENABLE_GETYPE_64BIT_DEFINE */

typedef float                  f32;
typedef double                 f64;



/****************************************************************
 *                                                              *
 * OS       :                                                   *
 * Compiler :   C30                                             *
 * Version  :                                                   *
 *                                                              *
 ****************************************************************/
#elif defined(_TMS320C30)

typedef char                   s8;
typedef unsigned char          n8;
typedef short                  s16;
typedef unsigned short         n16;
typedef long                   s32;
typedef unsigned long          n32;

#ifdef ENABLE_GETYPE_64BIT_DEFINE

#error 64bit GEtypes not defined for this platform.

#endif /* ENABLE_GETYPE_64BIT_DEFINE */

typedef float                  f32;
typedef double                 f64;



/****************************************************************
 *                                                              *
 * Default Section                                              *
 *                                                              *
 * The default section conditionaly excludes 64bit GEtypes      *
 * (n64, s64) based on DISABLE_GETYPE_64BIT_DEFINE.             *
 *                                                              *
 ****************************************************************/
#else /*  anything else goes here... */

#ifdef __GNUC__
#include <sys/types.h>
#endif

typedef char                   s8;
typedef unsigned char          n8;
typedef short                  s16;
typedef unsigned short         n16;
typedef long                   s32;
typedef unsigned long          n32;

#if !defined(DISABLE_GETYPE_64BIT_DEFINE)

typedef long long int          s64;
typedef unsigned long long int n64;

#endif /* DISABLE_GETYPE_64BIT_DEFINE */

typedef float                  f32;
typedef double                 f64;


#define GE_POINTER_SIZE 4


/* define missing ANSI types: */
typedef unsigned int           uintptr_t;
typedef int                    intptr_t;

typedef s8                     int_least8_t;
typedef n8                     uint_least8_t;
typedef s16                    int_least16_t;
typedef n16                    uint_least16_t;
typedef s32                    int_least32_t;
typedef n32                    uint_least32_t;

typedef int                    int_fast8_t;
typedef unsigned int           uint_fast8_t;
typedef int                    int_fast16_t;
typedef unsigned int           uint_fast16_t;
typedef int                    int_fast32_t;
typedef unsigned int           uint_fast32_t;


#if !defined(DISABLE_GETYPE_64BIT_DEFINE)

typedef s64                    int_least64_t;
typedef n64                    uint_least64_t;

typedef s64                    int_fast64_t;
typedef n64                    uint_fast64_t;
typedef s64                    intmax_t;
typedef n64                    uintmax_t;

#else

typedef s32                    intmax_t;
typedef n32                    uintmax_t;

#endif /* DISABLE_GETYPE_64BIT_DEFINE */

#endif /* defined(_WINDOWS) */

#endif /* GEtypes_INCL */
