/**<PrintImageFileHeaderSummary>**

     Title: PRINT IMAGEFILE HEADER SUMMARY

    Author: R Birn, NIH

   History: 02_01_06 - lrf Return dimensions string to use for to3d
          : 02_11_22 - rjt Return header information in subsections
*/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include "toolsdata.h"
#include "imagedb12x.h"
#include "rdbm12x.h"

/* ----------------------- */
int swap_2( char *arR, int nr )
/* ----------------------- */
{
   int i;
   char ctmp;

   for (i=0; i < nr; i+=2){
     ctmp     = arR[i+1];
     arR[i+1] = arR[i];
     arR[i]   = ctmp;
   }
}

/* ----------------------- */
int swap_4( char *arR, int nr )
/* ----------------------- */
{
   int i;
   char ctmp2, ctmp3;

   for (i=0; i < nr; i+=4){
     ctmp3    = arR[i+3];
     ctmp2    = arR[i+2];
     arR[i+2] = arR[i+1];
     arR[i+3] = arR[i];
     arR[i]   = ctmp3;
     arR[i+1] = ctmp2;
   }
}


/******
*
* printExamHeader( EXAMDATATYPE *)
*
* This function prints the contents of the exam data structure to
* the supplied file. 
*
******/

void printExamHeader( EXAMDATATYPE *hr_ex) {
 

   printf("\n################### EXAM HEADER #########################\n");
   
   printf("ex_suid:%s\n", hr_ex->ex_suid);             /*Suite ID for this Exam*/
   printf("ex_uniq:%i\n", hr_ex->ex_uniq);             /*The Make-Unique Flag*/
   printf("ex_diskid:%c\n", hr_ex->ex_diskid);         /*Disk ID for this Exam*/

   printf("ex_no:%i\n", hr_ex->ex_no);                  /*Exam Number*/
   printf("hospname:%s\n", hr_ex->hospname);           /*Hospital Name*/

   printf("detect:%i\n", hr_ex->detect);               /*Detector Type*/

   printf("numcells:%ld\n", hr_ex->numcells);          /*Number of cells in det*/
   printf("zerocell:%f\n", hr_ex->zerocell);           /*Cell number at theta*/
   printf("cellspace:%f\n", hr_ex->cellspace);         /*Cell spacing*/
   printf("srctodet:%f\n", hr_ex->srctodet);           /*Distance from source to detector*/
   printf("srctoiso:%f\n", hr_ex->srctoiso);           /*Distance from source to iso*/
   printf("tubetyp:%i\n", hr_ex->tubetyp);             /*Tube type*/
   printf("dastyp:%i\n", hr_ex->dastyp);               /*DAS type*/
   printf("num_dcnk:%i\n", hr_ex->num_dcnk);           /*Number of Decon Kernals*/
   printf("dcn_len:%i\n", hr_ex->dcn_len);             /*Number of elements in a Decon Kernal*/
   printf("dcn_density:%i\n", hr_ex->dcn_density);     /*Decon Kernal density*/
   printf("dcn_stepsize:%i\n", hr_ex->dcn_stepsize);   /*Decon Kernal stepsize*/
   printf("dcn_shiftcnt:%i\n", hr_ex->dcn_shiftcnt);   /*Decon Kernal Shift Count*/

   printf("magstrength:%ld\n", hr_ex->magstrength);    /*Magnet strength (in gauss)*/
   printf("patid:%s\n", hr_ex->patid);                 /*Patient ID for this Exam*/
   printf("patname:%s\n", hr_ex->patname);             /*Patientsda Name*/
   printf("patage:%i\n", hr_ex->patage);               /*Patient Age (years, months or days)*/
   printf("patian:%i\n", hr_ex->patian);               /*Patient Age Notation*/
   printf("patsex:%i\n", hr_ex->patsex);               /*Patient Sex*/
   printf("patweight:%i\n", hr_ex->patweight);        /*Patient Weight*/
   printf("trauma:%i\n", hr_ex->trauma);               /*Trauma Flag*/
   printf("hist:%s\n", hr_ex->hist);                   /*Patient History*/
   printf("reqnum:%s\n", hr_ex->reqnum);               /*Requisition Number*/
   printf("ex_datetime:%ld\n", hr_ex->ex_datetime);    /*Exam date/time stamp*/
   printf("refphy:%s\n", hr_ex->refphy);               /*Referring Physician*/
   printf("diagrad:%s\n", hr_ex->diagrad);             /*Diagnostician/Radiologist*/
   printf("op:%s\n", hr_ex->op);                       /*Operator*/
   printf("ex_desc:%s\n", hr_ex->ex_desc);             /*Exam Description*/
   printf("ex_typ:%s\n", hr_ex->ex_typ);               /*Exam Type*/
   printf("ex_format:%i\n", hr_ex->ex_format);         /*Exam Format*/

   printf("firstaxtime:\n");                           /*Start time(secs) of first axial in exam*/
   printf("ex_sysid:%s\n", hr_ex->ex_sysid);           /*Creator Suite and Host*/

   printf("ex_lastmod:%ld\n", hr_ex->ex_lastmod);      /*Date/Time of Last Change*/
   printf("protocolflag:%i\n", hr_ex->protocolflag);   /*Non-Zero indicates Protocol Exam*/
   printf("ex_alloc_key:%s\n", hr_ex->ex_alloc_key);   /*Process that allocated this record*/

   printf("ex_delta_cnt:%ld\n", hr_ex->ex_delta_cnt);  /*Indicates number of updates to header*/
   printf("ex_verscre:%s\n", hr_ex->ex_verscre);       /*Genesis Version - Created*/
   printf("ex_verscur:%s\n", hr_ex->ex_verscur);       /*Genesis Version - Now*/
   printf("ex_checksum:%lu\n", hr_ex->ex_checksum);     /*Exam Record Checksum*/
   printf("ex_complete:%ld\n", hr_ex->ex_complete);    /*Exam Complete Flag*/
   printf("ex_seriesct:%ld\n", hr_ex->ex_seriesct);    /*Last Series Number Used*/
   printf("ex_numarch:%ld\n", hr_ex->ex_numarch);      /*Number of Series Archived*/
   printf("ex_numseries:%ld\n", hr_ex->ex_numseries);  /*Number of Series Existing*/

   printf("ex_numunser:%ld\n", hr_ex->ex_numunser);    /*Number of Unstored Series*/

   printf("ex_toarchcnt:%ld\n", hr_ex->ex_toarchcnt);  /*Number of Unarchived Series*/

   printf("ex_prospcnt:%ld\n", hr_ex->ex_prospcnt);    /*Number of Prospective/Scout Series*/

   printf("ex_modelnum:%ld\n", hr_ex->ex_modelnum);    /*Last Model Number used*/
   printf("ex_modelcnt:%ld\n", hr_ex->ex_modelcnt);    /*Number of ThreeD Models*/

   //printf("ex_stat:%i\n", hr_ex->ex_stat);             /*Patient Status*/
   printf("uniq_sys_id:%s\n", hr_ex->uniq_sys_id);     /*Unique System ID*/
   printf("service_id:%s\n", hr_ex->service_id);       /*Unique Service ID*/
   printf("mobile_loc:%s\n", hr_ex->mobile_loc);       /*Mobile Location Number*/
   printf("study_uid:%s\n", hr_ex->study_uid);         /*Study Entity Unique ID*/
   printf("study_status:%i\n", hr_ex->study_status);   /*indicates if study has complete info(DICOM/genesis)*/
    

}

/* end printExamHeader */


/******
*
* printSeriesHeader ( SERIESDATATYPE *, FILE * )
*
* This function prints the contents of the Series header structure to
* the supplied file. 
*
******/

void printSeriesHeader( SERIESDATATYPE *hr_se){



   printf("\n################### SERIES HEADER #########################\n");
   
   printf("se_suid:%s\n", hr_se->se_suid);                  /*Suite ID for this Series*/
   printf("se_uniq:%i\n", hr_se->se_uniq);                  /*The Make-Unique Flag*/
   printf("se_diskid:%c\n", hr_se->se_diskid);              /*Disk ID for this Series*/

   printf("se_exno:%i\n", hr_se->se_exno);                  /*Exam Number*/
   printf("se_no :%i\n", hr_se->se_no );                    /*Series Number*/
   printf("se_datetime:%ld\n", hr_se->se_datetime);         /*Allocation Series Data/Time stamp*/
   printf("se_actual_dt:%ld\n", hr_se->se_actual_dt);       /*Actual Series Data/Time stamp*/
   printf("se_desc:%s\n", hr_se->se_desc);                  /*Series Description*/
   printf("pr_sysid:%s\n", hr_se->pr_sysid);                /*Primary Receiver Suite and Host*/
   printf("pansysid:%s\n", hr_se->pansysid);                /*Archiver Suite and Host*/
   printf("se_typ :%i\n", hr_se->se_typ );                  /*Series Type*/
   printf("se_source :%i\n", hr_se->se_source );            /*Series from which prescribed*/
   printf("se_plane :%i\n", hr_se->se_plane );              /*Most-like Plane (for L/S)*/
   printf("scan_type :%i\n", hr_se->scan_type );            /*Scout or Axial (for CT)*/
   printf("position:%ld\n", hr_se->position);               /*Patient Position*/
   printf("entry:%ld\n", hr_se->entry);                     /*Patient Entry*/
   printf("anref:%s\n", hr_se->anref);                      /*Anatomical reference*/

   printf("lmhor:%f\n", hr_se->lmhor);                      /*Horizontal Landmark*/
   printf("prtcl:%s\n", hr_se->prtcl);                      /*Scan Protocol Name*/

   printf("se_contrast :%i\n", hr_se->se_contrast );        /*Non-zero if > 0 image used contrast(L/S)*/
   printf("start_ras:%c\n", hr_se->start_ras);              /*RAS letter for first scan location (L/S)*/

   printf("start_loc:%f\n", hr_se->start_loc);              /*First scan location (L/S)*/
   printf("end_ras:%c\n", hr_se->end_ras);                  /*RAS letter for last scan location (L/S)*/

   printf("end_loc:%f\n", hr_se->end_loc);                  /*Last scan location (L/S)*/
   printf("se_pseq :%i\n", hr_se->se_pseq );                /*Last Pulse Sequence Used (L/S)*/
   printf("se_sortorder :%i\n", hr_se->se_sortorder );      /*Image Sort Order (L/S)*/
   printf("se_lndmrkcnt:%ld\n", hr_se->se_lndmrkcnt);       /*Landmark Counter*/
   printf("se_nacq :%i\n", hr_se->se_nacq );                /*Number of Acquisitions*/
   printf("xbasest :%i\n", hr_se->xbasest );                /*Starting number for baselines*/
   printf("xbaseend:%i\n", hr_se->xbaseend);                /*Ending number for baselines*/
   printf("xenhst:%i\n", hr_se->xenhst);                    /*Starting number for enhanced scans*/
   printf("xenhend:%i\n", hr_se->xenhend);                  /*Ending number for enhanced scans*/

   printf("se_lastmod:%ld\n", hr_se->se_lastmod);           /*Date/Time of Last Change*/
   printf("se_alloc_key:%s\n", hr_se->se_alloc_key);        /*Process that allocated this record*/

   printf("se_delta_cnt:%ld\n", hr_se->se_delta_cnt);       /*Indicates number of updates to header*/
   printf("se_verscre:%s\n", hr_se->se_verscre);   /*Genesis Version - Created*/
   printf("se_verscur:%s\n", hr_se->se_verscur);   /*Genesis Version - Now*/
   printf("se_pds_a:%f\n", hr_se->se_pds_a);                /*PixelData size - as stored*/
   printf("se_pds_c:%f\n", hr_se->se_pds_c);                /*PixelData size - Compressed*/
   printf("se_pds_u:%f\n", hr_se->se_pds_u);                /*PixelData size - UnCompressed*/
   printf("se_checksum:%lu\n", hr_se->se_checksum);          /*Series Record checksum*/
   printf("se_complete:%ld\n", hr_se->se_complete);         /*Series Complete Flag*/
   printf("se_numarch:%ld\n", hr_se->se_numarch);           /*Number of Images Archived*/
   printf("se_imagect:%ld\n", hr_se->se_imagect);           /*Last Image Number Used*/
   printf("se_numimages:%ld\n", hr_se->se_numimages);       /*Number of Images Existing*/

   printf("se_numunimg:%ld\n", hr_se->se_numunimg);         /*Number of Unstored Images*/

   printf("se_toarchcnt:%ld\n", hr_se->se_toarchcnt);       /*Number of Unarchived Images*/

   printf("echo1_alpha:%f\n", hr_se->echo1_alpha);          /*Echo 1 Alpha Value*/
   printf("echo1_beta:%f\n", hr_se->echo1_beta);            /*Echo 1 Beta Value*/
   printf("echo1_window:%i\n", hr_se->echo1_window);        /*Echo 1 Window Value*/
   printf("echo1_level:%i\n", hr_se->echo1_level);          /*Echo 1 Level Value*/
   printf("echo2_alpha:%f\n", hr_se->echo2_alpha);          /*Echo 2 Alpha Value*/
   printf("echo2_beta:%f\n", hr_se->echo2_beta);            /*Echo 2 Beta Value*/
   printf("echo2_window:%i\n", hr_se->echo2_window);        /*Echo 2 Window Value*/
   printf("echo2_level:%i\n", hr_se->echo2_level);          /*Echo 2 Level Value*/
   printf("echo3_alpha:%f\n", hr_se->echo3_alpha);          /*Echo 3 Alpha Value*/
   printf("echo3_beta:%f\n", hr_se->echo3_beta);            /*Echo 3 Beta Value*/
   printf("echo3_window:%i\n", hr_se->echo3_window);        /*Echo 3 Window Value*/
   printf("echo3_level:%i\n", hr_se->echo3_level);          /*Echo 3 Level Value*/
   printf("echo4_alpha:%f\n", hr_se->echo4_alpha);          /*Echo 4 Alpha Value*/
   printf("echo4_beta:%f\n", hr_se->echo4_beta);            /*Echo 4 Beta Value*/
   printf("echo4_window:%i\n", hr_se->echo4_window);        /*Echo 4 Window Value*/
   printf("echo4_level:%i\n", hr_se->echo4_level);          /*Echo 4 Level Value*/
   printf("echo5_alpha:%f\n", hr_se->echo5_alpha);          /*Echo 5 Alpha Value*/
   printf("echo5_beta:%f\n", hr_se->echo5_beta);            /*Echo 5 Beta Value*/
   printf("echo5_window:%i\n", hr_se->echo5_window);        /*Echo 5 Window Value*/
   printf("echo5_level:%i\n", hr_se->echo5_level);          /*Echo 5 Level Value*/
   printf("echo6_alpha:%f\n", hr_se->echo6_alpha);          /*Echo 6 Alpha Value*/
   printf("echo6_beta:%f\n", hr_se->echo6_beta);            /*Echo 6 Beta Value*/
   printf("echo6_window:%i\n", hr_se->echo6_window);        /*Echo 6 Window Value*/
   printf("echo6_level:%i\n", hr_se->echo6_level);          /*Echo 6 Level Value*/
   printf("echo7_alpha:%f\n", hr_se->echo7_alpha);          /*Echo 7 Alpha Value*/
   printf("echo7_beta:%f\n", hr_se->echo7_beta);            /*Echo 7 Beta Value*/
   printf("echo7_window:%i\n", hr_se->echo7_window);        /*Echo 7 Window Value*/
   printf("echo7_level:%i\n", hr_se->echo7_level);          /*Echo 7 Level Value*/
   printf("echo8_alpha:%f\n", hr_se->echo8_alpha);          /*Echo 8 Alpha Value*/
   printf("echo8_beta:%f\n", hr_se->echo8_beta);            /*Echo 8 Beta Value*/
   printf("echo8_window:%i\n", hr_se->echo8_window);        /*Echo 8 Window Value*/
   printf("echo8_level:%i\n", hr_se->echo8_level);          /*Echo 8 Level Value*/
   printf("series_uid:%s\n", hr_se->series_uid);            /*Series Entity Unique ID*/
   printf("landmark_uid:%s\n", hr_se->landmark_uid);        /*Landmark Unique ID*/
   printf("equipmnt_uid:%s\n", hr_se->equipmnt_uid);        /*Equipment Unique ID*/

}


/* end printSeriesHeader */


/******
*
* printMRImageHeader ( MRIMAGEDATATYPE *, FILE * )
*
* This function prints the contents of the Series header structure to
* the supplied file. 
*
******/

void printMRImageHeader( MRIMAGEDATATYPE *hr_im) {

   printf("\n################### IMAGE HEADER #########################\n");

   printf("psd_iname:%s\n",hr_im->psd_iname);		 /*PSD name from inside PSD*/
   
   printf("im_suid:%s\n",hr_im->im_suid);		 /*Suite id for this image*/
   printf("im_uniq:%i\n",hr_im->im_uniq);		 /*The Make-Unique Flag*/
   printf("im_diskid:%c\n",hr_im->im_diskid);	         /*Disk ID for this Image*/
   printf("im_exno:%i\n",hr_im->im_exno);		 /*Exam number for this image*/
   printf("im_seno:%i\n",hr_im->im_seno);		 /*Series Number for this image*/
   printf("im_no:%i\n",hr_im->im_no);			 /*Image Number*/

   printf("im_datetime:%ld\n",hr_im->im_datetime);       /*Allocation Image date/time stamp*/
   printf("im_actual_dt:%ld\n",hr_im->im_actual_dt);     /*Actual Image date/time stamp*/
   printf("sctime:%f\n",hr_im->sctime); 		 /*Duration of scan*/
   printf("slthick:%f\n",hr_im->slthick);		 /*Slice Thickness (mm)*/
   printf("imatrix_X:%i\n",hr_im->imatrix_X);	         /*Image matrix size - X*/
   printf("imatrix_Y:%i\n",hr_im->imatrix_Y);	         /*Image matrix size - Y*/
   printf("dfov:%f\n",hr_im->dfov);			 /*Display field of view - X (mm)*/
   printf("dfov_rect:%f\n",hr_im->dfov_rect);	         /*Display field of view - Y (if different)*/
   printf("dim_X:%f\n",hr_im->dim_X);			 /*Image dimension - X*/
   printf("dim_Y:%f\n",hr_im->dim_Y);			 /*Image dimension - Y*/
   printf("pixsize_X:%f\n",hr_im->pixsize_X);	         /*Image pixel size - X*/
   printf("pixsize_Y:%f\n",hr_im->pixsize_Y);	         /*Image pixel size - Y*/
   printf("pdid:%s\n",hr_im->pdid);			 /*Pixel Data ID*/
   printf("contrastIV:%s\n",hr_im->contrastIV); 	 /*IV Contrast Agent*/
   printf("contrastOral:%s\n",hr_im->contrastOral);	 /*Oral Contrast Agent*/
   printf("contmode:%i\n",hr_im->contmode);		 /*Image Contrast Mode*/
   printf("serrx:%i\n",hr_im->serrx);			 /*Series from which prescribed*/
   printf("imgrx:%i\n",hr_im->imgrx);			 /*Image from which prescribed*/
   printf("screenformat:%i\n",hr_im->screenformat);      /*Screen Format(8/16 bit)*/
   printf("plane:%i\n",hr_im->plane);			 /*Plane Type*/

   printf("scanspacing:%f\n",hr_im->scanspacing);        /*Spacing between scans (mm?)*/
   printf("im_compress:%i\n",hr_im->im_compress);        /*Image compression type for allocation*/
   printf("im_scouttype:%i\n",hr_im->im_scouttype);      /*Scout Type (AP or lateral)*/
   printf("loc_ras:%c\n", hr_im->loc_ras);		 /*RAS letter of image location*/

   printf("loc:%f\n",hr_im->loc);			 /*Image location*/
   printf("ctr_R:%f\n",hr_im->ctr_R);			 /*Center R coord of plane image*/
   printf("ctr_A:%f\n",hr_im->ctr_A);			 /*Center A coord of plane image*/
   printf("ctr_S:%f\n",hr_im->ctr_S);			 /*Center S coord of plane image*/
   printf("norm_R:%f\n",hr_im->norm_R); 		 /*Normal R coord*/
   printf("norm_A:%f\n",hr_im->norm_A); 		 /*Normal A coord*/
   printf("norm_S:%f\n",hr_im->norm_S); 		 /*Normal S coord*/
   printf("tlhc_R:%f\n",hr_im->tlhc_R); 		 /*R Coord of Top Left Hand Corner*/
   printf("tlhc_A:%f\n",hr_im->tlhc_A); 		 /*A Coord of Top Left Hand Corner*/
   printf("tlhc_S:%f\n",hr_im->tlhc_S); 		 /*S Coord of Top Left Hand Corner*/
   printf("trhc_R:%f\n",hr_im->trhc_R); 		 /*R Coord of Top Right Hand Corner*/
   printf("trhc_A:%f\n",hr_im->trhc_A); 		 /*A Coord of Top Right Hand Corner*/
   printf("trhc_S:%f\n",hr_im->trhc_S); 		 /*S Coord of Top Right Hand Corner*/
   printf("brhc_R:%f\n",hr_im->brhc_R); 		 /*R Coord of Bottom Right Hand Corner*/
   printf("brhc_A:%f\n",hr_im->brhc_A); 		 /*A Coord of Bottom Right Hand Corner*/
   printf("brhc_S:%f\n",hr_im->brhc_S); 		 /*S Coord of Bottom Right Hand Corner*/
   printf("forimgrev:%s\n",hr_im->forimgrev);		 /*Foreign Image Revision*/
   printf("tr:%ld\n",hr_im->tr);			 /*Pulse repetition time(usec)*/
   printf("ti:%ld\n",hr_im->ti);			 /*Pulse inversion time(usec)*/
   printf("te:%ld\n",hr_im->te);			 /*Pulse echo time(usec)*/
   printf("te2:%ld\n",hr_im->te2);			 /*Second echo echo (usec)*/
   printf("numecho:%i\n",hr_im->numecho);		 /*Number of echoes*/
   printf("echonum:%i\n",hr_im->echonum);		 /*Echo Number*/
   printf("tbldlta:%f\n",hr_im->tbldlta);		 /*Table Delta*/
   printf("nex:%f\n",hr_im->nex);			 /*Number of Excitations*/
   printf("contig:%i\n",hr_im->contig); 		 /*Continuous Slices Flag*/
   printf("hrtrate:%i\n",hr_im->hrtrate);		 /*Cardiac Heart Rate (bpm)*/
   printf("tdel:%ld\n",hr_im->tdel);			 /*Delay time after trigger (msec)*/
   printf("saravg:%f\n",hr_im->saravg); 		 /*Average SAR*/
   printf("sarpeak:%f\n",hr_im->sarpeak);		 /*Peak SAR*/
   printf("monsar:%i\n",hr_im->monsar); 		 /*Monitor SAR flag*/
   printf("trgwindow:%i\n",hr_im->trgwindow);	         /*Trigger window (% of R-R interval)*/
   printf("reptime:%f\n",hr_im->reptime);		 /*Cardiac repetition time*/
   printf("imgpcyc:%i\n",hr_im->imgpcyc);		 /*Images per cardiac cycle*/
   printf("xmtgain:%i\n",hr_im->xmtgain);		 /*Actual Transmit Gain (.1 db)*/
   printf("rcvgain1:%i\n",hr_im->rcvgain1);		 /*Actual Receive Gain Analog (.1 db)*/
   printf("rcvgain2:%i\n",hr_im->rcvgain2);		 /*Actual Receive Gain Digital (.1 db)*/
   printf("mr_flip:%i\n",hr_im->mr_flip);		 /*Flip Angle for GRASS scans (deg.)*/

   printf("mindat:%ld\n",hr_im->mindat);		 /*Minimum Delay after Trigger (uSec)*/
   printf("cphase:%i\n",hr_im->cphase); 		 /*Total Cardiac Phase prescribed*/
   printf("swappf:%i\n",hr_im->swappf); 		 /*Swap Phase/Frequency Axis*/
   printf("pauseint:%i\n",hr_im->pauseint);		 /*Pause Interval (slices)*/

   printf("pausetime:%f\n",hr_im->pausetime);	       /*Pause Time*/
   printf("obplane:%ld\n",hr_im->obplane);	       /*Oblique Plane*/
   printf("slocfov:%ld\n",hr_im->slocfov);	       /*Slice Offsets on Freq axis*/
   printf("xmtfreq:%ld\n",hr_im->xmtfreq);	       /*Center Frequency (0.1 Hz)*/
   printf("autoxmtfreq:%ld\n",hr_im->autoxmtfreq);     /*Auto Center Frequency (0.1 Hz)*/
   printf("autoxmtgain:%i\n",hr_im->autoxmtgain);      /*Auto Transmit Gain (0.1 dB)*/
   printf("prescan_r1:%i\n",hr_im->prescan_r1);        /*PreScan R1 - Analog*/
   printf("prescan_r2:%i\n",hr_im->prescan_r2);        /*PreScan R2 - Digital*/

   printf("user_bitmap:%ld\n",hr_im->user_bitmap);     /*Bitmap defining user CVs*/
   printf("cenfreq:%i\n",hr_im->cenfreq);	       /*Center Frequency Method*/
   printf("imode:%i\n",hr_im->imode);		       /*Imaging Mode*/
   printf("iopt:%ld\n",hr_im->iopt);		       /*Imaging Options*/
   printf("pseq:%i\n",hr_im->pseq);		       /*Pulse Sequence*/
   printf("pseqmode:%i\n",hr_im->pseqmode);	       /*Pulse Sequence Mode*/
   printf("psdname:%s\n",hr_im->psdname);	       /*Pulse Sequence Name*/

   printf("psd_datetime:%ld\n",hr_im->psd_datetime);   /*PSD Creation Date and Time*/
   printf("psd_iname:%s\n",hr_im->psd_iname);	       /*PSD name from inside PSD*/

   printf("ctyp:%i\n",hr_im->ctyp);		       /*Coil Type*/
   printf("cname:%s\n",hr_im->cname);		       /*Coil Name*/

   printf("surfctyp:%i\n",hr_im->surfctyp);	       /*Surface Coil Type*/
   printf("surfcext:%i\n",hr_im->surfcext);	       /*Extremity Coil Flag*/

   printf("rawrunnum:%ld\n",hr_im->rawrunnum);         /*RawData Run Number*/
   printf("cal_fldstr:%lu\n",hr_im->cal_fldstr);       /*Calibrated Field Strength (x10 uGauss)*/
   printf("supp_tech:%i\n",hr_im->supp_tech);	       /*SAT fat/water/none*/

   printf("vbw:%f\n",hr_im->vbw);		       /*Variable Bandwidth (Hz)*/
   printf("slquant:%i\n",hr_im->slquant);	       /*Number of slices in this scan group*/
   printf("gpre:%i\n",hr_im->gpre);		       /*Graphically prescribed*/
   printf("intr_del:%ld\n",hr_im->intr_del);	       /*Interimage/interloc delay (uSec)*/
   printf("user0:%f\n",hr_im->user0);		       /*User Variable 0*/
   printf("user1:%f\n",hr_im->user1);		       /*User Variable 1*/
   printf("user2:%f\n",hr_im->user2);		       /*User Variable 2*/
   printf("user3:%f\n",hr_im->user3);		       /*User Variable 3*/
   printf("user4:%f\n",hr_im->user4);		       /*User Variable 4*/
   printf("user5:%f\n",hr_im->user5);		       /*User Variable 5*/
   printf("user6:%f\n",hr_im->user6);		       /*User Variable 6*/
   printf("user7:%f\n",hr_im->user7);		       /*User Variable 7*/
   printf("user8:%f\n",hr_im->user8);		       /*User Variable 8*/
   printf("user9:%f\n",hr_im->user9);		       /*User Variable 9*/
   printf("user10:%f\n",hr_im->user10); 	       /*User Variable 10*/
   printf("user11:%f\n",hr_im->user11); 	       /*User Variable 11*/
   printf("user12:%f\n",hr_im->user12); 	       /*User Variable 12*/
   printf("user13:%f\n",hr_im->user13); 	       /*User Variable 13*/
   printf("user14:%f\n",hr_im->user14); 	       /*User Variable 14*/
   printf("user15:%f\n",hr_im->user15); 	       /*User Variable 15*/
   printf("user16:%f\n",hr_im->user16); 	       /*User Variable 16*/
   printf("user17:%f\n",hr_im->user17); 	       /*User Variable 17*/
   printf("user18:%f\n",hr_im->user18); 	       /*User Variable 18*/
   printf("user19:%f\n",hr_im->user19); 	       /*User Variable 19*/
   printf("user20:%f\n",hr_im->user20); 	       /*User Variable 20*/
   printf("user21:%f\n",hr_im->user21); 	       /*User Variable 21*/
   printf("user22:%f\n",hr_im->user22); 	       /*User Variable 22*/
   printf("user23:%f\n",hr_im->user23); 	       /*Projection Angle*/
   printf("user24:%f\n",hr_im->user24); 	       /*Concat Sat Type Flag*/
   printf("im_alloc_key:%s\n",hr_im->im_alloc_key);

   printf("im_lastmod:%ld\n",hr_im->im_lastmod);       /*Date/Time of Last Change*/
   printf("im_verscre:%s\n",hr_im->im_verscre);        /*Genesis Version - Created*/
   printf("im_verscur:%s\n",hr_im->im_verscur);        /*Genesis Version - Now*/
   printf("im_pds_a:%ld\n",hr_im->im_pds_a);	       /*PixelData size - as stored*/
   printf("im_pds_c:%ld\n",hr_im->im_pds_c);	       /*PixelData size - Compressed*/
   printf("im_pds_u:%ld\n",hr_im->im_pds_u);	       /*PixelData size - UnCompressed*/
   printf("im_checksum:%ld\n",hr_im->im_checksum);     /*AcqRecon record checksum*/
   printf("im_archived:%ld\n",hr_im->im_archived);     /*Image Archive Flag*/
   printf("im_complete:%ld\n",hr_im->im_complete);     /*Image Complete Flag*/
   printf("satbits:%i\n",hr_im->satbits);	       /*Bitmap of SAT selections*/
   printf("scic:%i\n",hr_im->scic);		       /*Surface Coil Intensity Correction Flag*/
   printf("satxloc1:%i\n",hr_im->satxloc1);	       /*R-side SAT pulse loc rel to lndmrk*/
   printf("satxloc2:%i\n",hr_im->satxloc2);	       /*L-side SAT pulse loc rel to lndmrk*/
   printf("satyloc1:%i\n",hr_im->satyloc1);	       /*A-side SAT pulse loc rel to lndmrk*/
   printf("satyloc2:%i\n",hr_im->satyloc2);	       /*P-side SAT pulse loc rel to lndmrk*/
   printf("satzloc1:%i\n",hr_im->satzloc1);	       /*S-side SAT pulse loc rel to lndmrk*/
   printf("satzloc2:%i\n",hr_im->satzloc2);	       /*I-side SAT pulse loc rel to lndmrk*/
   printf("satxthick:%i\n",hr_im->satxthick);	       /*Thickness of X-axis SAT pulse*/
   printf("satythick:%i\n",hr_im->satythick);	       /*Thickness of Y-axis SAT pulse*/
   printf("satzthick:%i\n",hr_im->satzthick);	       /*Thickness of Z-axis SAT pulse*/
   printf("flax:%i\n",hr_im->flax);		       /*Phase contrast flow axis*/
   printf("venc:%i\n",hr_im->venc);		       /*Phase contrast velocity encoding*/
   printf("thk_disclmr:%i\n",hr_im->thk_disclmr);      /*Slice Thickness*/
   printf("ps_flag:%i\n",hr_im->ps_flag);	       /*Auto/Manual Prescan flag*/
   printf("ps_status:%i\n",hr_im->ps_status);	       /*Bitmap of changed values*/
   printf("image_type:%i\n",hr_im->image_type);        /*Magnitude, Phase, Imaginary, or Real*/
   printf("vas_collapse:%i\n",hr_im->vas_collapse);    /*Collapse Image*/
   printf("proj_alg:%i\n",hr_im->proj_alg);	       /*Projection Algorithm*/
   printf("proj_name:%s\n",hr_im->proj_name);	       /*Projection Algorithm Name*/

   printf("x_axis_rot:%f\n",hr_im->x_axis_rot);        /*X Axis Rotation*/
   printf("y_axis_rot:%f\n",hr_im->y_axis_rot);        /*Y Axis Rotation*/
   printf("z_axis_rot:%f\n",hr_im->z_axis_rot);        /*Z Axis Rotation*/
   printf("thresh_min1:%ld\n",hr_im->thresh_min1);     /*Lower Range of Pixels 1*/
   printf("thresh_max1:%ld\n",hr_im->thresh_max1);     /*Upper Range of Pixels 1*/
   printf("thresh_min2:%ld\n",hr_im->thresh_min2);     /*Lower Range of Pixels 2*/
   printf("thresh_max2:%ld\n",hr_im->thresh_max2);     /*Upper Range of Pixels 2*/
   printf("echo_trn_len:%i\n",hr_im->echo_trn_len);    /*Echo Train Length for Fast Spin Echo*/
   printf("frac_echo:%i\n",hr_im->frac_echo);	       /*Fractional Echo - Effective TE Flag*/
   printf("prep_pulse:%i\n",hr_im->prep_pulse);        /*Preporatory Pulse Option*/
   printf("cphasenum:%i\n",hr_im->cphasenum);	       /*Cardiac Phase Number*/
   printf("var_echo:%i\n",hr_im->var_echo);	       /*Variable Echo Flag*/
   printf("ref_img:%c\n",hr_im->ref_img);	       /*Reference Image Field*/
   printf("sum_img:%c\n",hr_im->sum_img);	       /*Summary Image Field*/
   printf("img_window:%i\n",hr_im->img_window);        /*Window Value*/
   printf("img_level:%i\n",hr_im->img_level);	       /*Level Value*/



   printf("slop_str_2:%s\n",hr_im->slop_str_2);        /*String Slop Field 2*/
   printf("scanactno:%i\n",hr_im->scanactno);	       /*Scan Acquisition Number*/
   printf("vasflags:%i\n",hr_im->vasflags);	       /*Magnitude Weighting Flag*/
   printf("vencscale:%f\n",hr_im->vencscale);	       /*Scale Weighted Venc*/
   printf("integrity:%i\n",hr_im->integrity);	       /*GE Image Integrity*/

   printf("fphase:%ld\n",hr_im->fphase);	       /*Number Of Phases*/
   printf("freq_dir:%i\n",hr_im->freq_dir);	       /*Frequency Direction*/
   printf("vas_mode:%i\n",hr_im->vas_mode);	       /*Vascular Mode*/
   printf("image_uid:%s\n",hr_im->image_uid);	       /*Image Unique ID*/
   printf("sop_uid:%s\n",hr_im->sop_uid);	       /*Service Obj Class Unique ID*/
   //printf("dont_use_1:%i\n",hr_im->dont_use_1);        /*This field is not used*/
   //printf("dont_use_2:%i\n",hr_im->dont_use_2);        /*This field is not used*/
   //printf("dont_use_3:%i\n",hr_im->dont_use_3);        /*This field is not used*/
   printf("pscopts:%i\n",hr_im->pscopts);	       /*bitmap of prescan options*/
   printf("asoffsetx:%i\n",hr_im->asoffsetx);	       /*gradient offset in X-direction*/
   printf("asoffsety:%i\n",hr_im->asoffsety);	       /*gradient offset in Y-direction*/
   printf("asoffsetz:%i\n",hr_im->asoffsetz);	       /*gradient offset in Z-direction*/
   printf("unoriginal:%i\n",hr_im->unoriginal);        /*identifies image as original or unoriginal*/
   printf("interleaves:%i\n",hr_im->interleaves);      /*number of EPI shots*/
   printf("effechospace:%i\n",hr_im->effechospace);    /*effective echo spacing for EPI*/
   printf("viewsperseg:%i\n",hr_im->viewsperseg);      /*views per segment*/
   printf("rbpm:%i\n",hr_im->rbpm);		       /*respiratory rate, breaths per min*/
   printf("rtpoint:%i\n",hr_im->rtpoint);	       /*respiratory trigger point as percent of max.*/
   printf("rcvrtype:%i\n",hr_im->rcvrtype);	       /*type of receiver used*/
   printf("dbdt:%f\n",hr_im->dbdt);		       /*peak rate of change of gradient field, tesla/sec*/
   printf("dbdtper:%f\n",hr_im->dbdtper);	       /*limit in units of percent of theoretical curve*/
   printf("estdbdtper:%f\n",hr_im->estdbdtper);        /*PSD estimated limit in units of percent*/
   printf("estdbdtts:%f\n",hr_im->estdbdtts);	       /*PSD estimated limit in Teslas/sec*/
   printf("saravghead:%f\n",hr_im->saravghead);        /*Avg head SAR*/
   printf("neg_scanspacing:%f\n",hr_im->neg_scanspacing);      /*Negative scan spacing for overlap slices*/
   printf("offsetfreq:%ld\n",hr_im->offsetfreq);	       /*Offset Frequency - Mag.Transfer*/
   printf("user_usage_tag:%lu\n",hr_im->user_usage_tag);       /*Defines how following user CVs are to be filled in*/
           					               /*Default value = 0x00000000*/
           					               /*GE range = hr_im->0x00000001 - 0x7fffffff*/
           					               /*Research = 0x80000000 - 0xffffffff*/
   printf("user_fill_mapMSW:%lu\n",hr_im->user_fill_mapMSW);   /*Define what process fills in the user CVs, ifcc or TIR*/
   printf("user_fill_mapLSW:%lu\n",hr_im->user_fill_mapLSW);   /*Define what process fills in the user CVs, ifcc or TIR*/
   printf("user25:%f\n",hr_im->user25); 	       /*User Variable 25*/
   printf("user26:%f\n",hr_im->user26); 	       /*User Variable 26*/
   printf("user27:%f\n",hr_im->user27); 	       /*User Variable 27*/
   printf("user28:%f\n",hr_im->user28); 	       /*User Variable 28*/
   printf("user29:%f\n",hr_im->user29); 	       /*User Variable 29*/
   printf("user30:%f\n",hr_im->user30); 	       /*User Variable 30*/
   printf("user31:%f\n",hr_im->user31); 	       /*User Variable 31*/
   printf("user32:%f\n",hr_im->user32); 	       /*User Variable 32*/
   printf("user33:%f\n",hr_im->user33); 	       /*User Variable 33*/
   printf("user34:%f\n",hr_im->user34); 	       /*User Variable 34*/
   printf("user35:%f\n",hr_im->user35); 	       /*User Variable 35*/
   printf("user36:%f\n",hr_im->user36); 	       /*User Variable 36*/
   printf("user37:%f\n",hr_im->user37); 	       /*User Variable 37*/
   printf("user38:%f\n",hr_im->user38); 	       /*User Variable 38*/
   printf("user39:%f\n",hr_im->user39); 	       /*User Variable 39*/
   printf("user40:%f\n",hr_im->user40); 	       /*User Variable 40*/
   printf("user41:%f\n",hr_im->user41); 	       /*User Variable 41*/
   printf("user42:%f\n",hr_im->user42); 	       /*User Variable 42*/
   printf("user43:%f\n",hr_im->user43); 	       /*User Variable 43*/
   printf("user44:%f\n",hr_im->user44); 	       /*User Variable 44*/
   printf("user45:%f\n",hr_im->user45); 	       /*User Variable 45*/
   printf("user46:%f\n",hr_im->user46); 	       /*User Variable 46*/
   printf("user47:%f\n",hr_im->user47); 	       /*User Variable 47*/
   printf("user48:%f\n",hr_im->user48); 	       /*User Variable 48*/



}
/* end printMRImageHeader */

void printACQTable( RDB_DATA_ACQ_TAB  *hr_acq) {
   int i;
   RDB_SLICE_INFO_ENTRY *dummy;
   printf("\n################### SLICE ACQ TABLE  #########################\n");
   for (i=0; i < RDB_MAX_SLICES; i++){
     dummy = &(hr_acq[0][i]);

     printf("count:%d\tpass_number:%i\tslice in pass:%i\n",i,dummy->pass_number,dummy->slice_in_pass);
     printf("gw_point1:%f\t%f\t%f\n",dummy->gw_point1[0],dummy->gw_point1[1],dummy->gw_point1[2]);
     printf("gw_point2:%f\t%f\t%f\n",dummy->gw_point2[0],dummy->gw_point2[1],dummy->gw_point2[2]);
     printf("gw_point3:%f\t%f\t%f\n\n",dummy->gw_point3[0],dummy->gw_point3[1],dummy->gw_point3[2]);

   }


}
/* end printACQTable */
