/*** bam.h  --  Include File for Bulk Acquisition Memory (BAM) Definitions
*
*    Description:
*
*        This include file contains the minimum BAM definitions needed 
*        for non-recon processes. 
*                                                  M. Krauski July, 2006
******************************************************************************/

#ifndef BAM_H
#define BAM_H

#include "GEtypes.h"

/* BAM Loader Memory Model Mask Values */
#define LOAD_BAM_FLAT_MODEL     (1 << 1)
#define LOAD_BAM_PAS_MODEL      (1 << 2)
#define LOAD_BAM_HOST_MODEL     (1 << 3)

/* BAM Loader Error Mask Values */
#define LOAD_BAM_MALLOC_ERROR       (1 << 1)
#define LOAD_BAM_FLAT_MODEL_ERROR   (1 << 2)
#define LOAD_BAM_PAS_MODEL_ERROR    (1 << 3)
#define LOAD_BAM_HOST_MODEL_ERROR   (1 << 4)

/* BAM Memory Models */
/* These are indicies into BamOpen()'s bam_models[] array */
/* Used as argument to BamSize() */
enum BAM_MEMORY_MODEL
{
    BAM_FLAT_MODEL = 0,
    BAM_PAS_MODEL,
    BAM_HOST_MODEL,
    NUM_BAM_MEMORY_MODELS
};
#define MAX_BAM_MEMORY_MODELS 3

/**
 * BAM Address typedef Definition.
 *
 * This type is intended to be used by remote processes that
 * need to access Recon memory.
 * 
 * WARNING!!! The name BAM_addr is miss leading. BAM is a 
 * virtualized memory space that can only be accessed via the
 * BAM Driver. The BAM_addr type is actually intended to hold
 * Recon memory addresses which are used by external processes
 * to read/write directly to those locations. (Typicaly using
 * tn_lib.) The use of the name BAM has been preserved for 
 * legacy purposes.
 */
typedef n64 BAM_addr;

#define BAM_NULL (BAM_ptr)0

/* BAM Pointer typedef Definition */
typedef uintptr_t BAM_ptr;
typedef size_t BAM_size;


typedef struct      /* pass this struct on ba_st() status calls */
{
    BAM_size t_size;        /* total size of BAM */
    BAM_size st_size;       /* amount of short term memory available */
    BAM_size lt_size;       /* amount of long-term memory in use */
    BAM_size lag;           /* last address given by BAMgr */
} BA_ST_BLOCK_TYPE;

int BamLoader( int argc, char** argv, unsigned int loadBamModelMask );

#endif /* BAM_H */

