/**********************************************************************
 * Copyright 2006 General Electric Company.  All rights reserved
 * 
 *     File Name:  coilIdDefs.h
 *
 *     Description:
 *
 *         This file contains the common coil ID definitions that are
 *         used on the Host and servers.
 *
 *     Release #  Date        Author         Changes
 *     ---------  ---------   -------------  ---------------------------
 *       20.0     05-Mar-06   Tom Haworth    Original
 *       20.0     30-Aug-07   Tom Haworth    Added MNS conversion values
 ******************************************************************** */

#ifndef _COIL_ID_DEFS_INCLUDE_
#define _COIL_ID_DEFS_INCLUDE_

/* Number of coils allowed per connector */
#define NUM_COILS_PER_CONNECTOR 4

/* Length of the serial ID of the coil, numeric array (not ASCII) */
#define COIL_ID_SERIAL_LENGTH   8

/* Length of the coil ID code and hash code ASCII strings */
#define COIL_ID_LENGTH          32

/* The actual string length is padded out to a multiple of 64 bits for
 * NULL termination and alignment */
#define COIL_ID_PADDED_LENGTH   (COIL_ID_LENGTH + 8)

/* Length of the coil name (physical coil description), with
 * NULL termination and alignment */
#define COIL_NAME_LENGTH        64

/* Coil ID code prefixes */
#define COIL_ID_PREFIX_RESEARCH "R000"
#define COIL_ID_PREFIX_PROTO    "P000"
#define COIL_ID_PREFIX_PROTO_GE "PG00"

/* MCRv tool Coil ID */
#define COIL_ID_MCRV_A_PORT "GEMSb1w9OwfkOl0g7HalfXMyey8hTkQI"
#define COIL_ID_MCRV_P_PORT "GEMSayvEZgAaPhfgaydfFQPJdWYmrH9a"

/* MCRv tool MUX select position */
#define A_PORT_OFFSET    25
#define P1_PORT_OFFSET   0           
#define P2_PORT_OFFSET   5
#define P3_PORT_OFFSET   10
#define P4_PORT_OFFSET   15
#define P5_PORT_OFFSET   20

/* MCRv 1-wire position */
#define A3_ONE_WIRE    3
#define A5_ONE_WIRE    5
#define A6_ONE_WIRE    6

/* MCRv Mux status mask */
#define A_PORT_MUX_STATUS_MASK 0x0F
#define Pn_PORT_MUX_STATUS_MASK 0x1F

/* Values for "connector" */
enum
{
    COIL_CONNECTOR_A,
    COIL_CONNECTOR_P1,
    COIL_CONNECTOR_P2,
    COIL_CONNECTOR_P3,
    COIL_CONNECTOR_P4,
    COIL_CONNECTOR_P5,

    /* Number of connectors */
    NUM_COIL_CONNECTORS,
    NUM_COIL_CONNECTORS_PADDED = 8 /* Padded for 64-bit alignment */
};

/* Values for "state" */
/* The UNKNOWN, DISCONNECTED, CONNECTED, and ID values are the only
 * ones used in communication with the Host.  The UNKNOWN value is
 * only used during initialization or when communication is down to
 * one of the subsystems.  The COMPLETE value is only used on the
 * server side when the connector info has been validated by the Host */
enum
{
    COIL_STATE_UNKNOWN,      /* State used when unknown (init or comm down) */
    COIL_STATE_DISCONNECTED, /* No coil connected */
    COIL_STATE_CONNECTED,    /* Coil connected */
    COIL_STATE_ID,           /* Coil ID read */
    COIL_STATE_COMPLETE,     /* Coil ID has been validated and is complete */

    /* Number of states */
    NUM_COIL_STATES
};

/* Values for "setValid" and "coilValid" */
enum
{
    COIL_INVALID,
    COIL_VALID,

    /* Number of values */
    NUM_COIL_VALID_VALUES
};

/* Values for "coilType" */
enum
{
    COIL_TYPE_TRANSMIT,
    COIL_TYPE_RECEIVE,

    /* Number of values */
    NUM_COIL_TYPE_VALUES
};

/* Values for "bodyTransmitEnable" */
enum
{
    BODY_TRANSMIT_DISABLE,
    BODY_TRANSMIT_ENABLE,

    /* Number of values */
    NUM_BODY_TRANSMIT_ENABLE_VALUES
};

/* Values for local transmit selects */
enum
{
    TRANSMIT_SELECT_NONE,   /* No local transmit selected (body) */
    TRANSMIT_SELECT_A,      /* Local transmit on connector A */
    TRANSMIT_SELECT_P1,     /* Local transmit on connector P1 */

    /* Number of values */
    NUM_TRANSMIT_SELECTS
};

/* Values for MNS converter select */
enum
{
    MNS_CONVERTER_SELECT_NONE = 0x00000000, /* No MNS conversion (bypass) */
    MNS_CONVERTER_SELECT_A    = 0x00000001, /* MNS conversion on A port */
    MNS_CONVERTER_SELECT_MASK = 0x00000001, /* Mask of all possible bits */
};

#endif /* _COIL_ID_DEFS_INCLUDE_ */

