/*$Id: epi.h,v 1.6 2007/09/21 00:33:32 ecwong Exp $*/
/**<epi.h>**************************************************

   Header file for epi.c

*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/fcntl.h>

#if RECON_FLAG==1
    #include "rdbm11x.h" 
#endif
#if RECON_FLAG==2
    #include "rdbm12x.h" 
#endif
#if RECON_FLAG==3
    #include "rdbm14x.h" 
#endif
#if RECON_FLAG==4
    #include "rdbm20x.h" 
#endif

#include "vec.h"

#define EPIPC_AV   0  /* Average all phase correction data per slice */
#define EPIPC_LIN  1  /* Linear interpolation through time */
#define EPIPC_EACH 2  /* Do phase correction per image */

#define EPI_IGNORE 2  /* Default points to ignore on each end of phase fit */
#define EPI2D 1    /* EPI->type */
#define GRASE3D 2  /* EPI->type */


void fftc();

typedef struct EPI {		/* echo planar imaging */

  int type ;        /* recon type: EPI2D or GRASE2D */
  /* Imaging params */

  int ndx,ndy ;	    /* x and y data dimensions */
  int oscan ;       /* # lines before center of K-space */
  int skip ;       /* # points skipped on ramps */
  int npix ;        /* output pixels, square images for now */
  int reps ;        /* repetitions */
  int ncoil ;       /* # coils */
  int nslice ;      /* # slices */
  int nzout ;       /* output slices for 3d */
  int necho ;       /* # echos */
  int frsize ;      /* # samples per raw data frame */
  int nl ;          /* # interleaves */
  int etl ;	        /* ymat/nleaf */
  int loopmode ;    /* 1=prep_inside 2=interleaves_inside */
  int nprep ;       /* # prep states */
  int npo2 ;        /* npix/2 */
  int coil_block ;  /* data offset per coil */
  int xcyc ;        /* # points per trapezoid in readout */
  int skipline ;    /* # redundant lines for phase correction - hardwired to 1 for now */
  int shift_fov ;   /* switch for off center FOV */
  int dopc ;        /* switch for phase correction */
  int gechoes ;     /* gradient echoes for grase */
  int rampsamp ;    /* ramp sampling */
  int ramp ;        /* number of points on ramp */
  int flat ;        /* number of points on flat */
  int nref ;        /* number of reference shots for 3dgrase - wired to 1 for now */
  int refftsize;    /* 2^n ft size for measuring shift of odd/even echoes */
  
  int xpad1,xpad2 ; /* zero padded points in X direction */
  int ypad1,ypad2 ; /* zero padded points in Y direction */
  int ignore ;      /* # pts to ignore on each for phase fits */
  
  float xoff, yoff; /* Spatial offsets from isocenter */

  int headfudge ;   /* fudge factor for byte offset into header, default = 0 */
  int headoffset ;  /* byte offset into header, default = headerbytes */

  float ***raw;		/* kspace data */
  float ***ref;		/* reference data */
  float ***rot;		/* rotation map */
  short int *ibuf;  /* input data buffer */
  
  float srad;       /* radius of gridding kernel */
  int sradi;        /* interpolation factor for gridding kernel */
  int *gsrc;        /* source array for rampsamp gridding */
  int *gdst;        /* destination array for rampsamp gridding */
  float *gwt;       /* weight array for rampsamp gridding */
  float *gf;        /* gridding function for rampsamp */
  int gcount;       /* count of gridding table */
  
  FILE *fin;

} EPI;

