/*<readrx.c>*********************************************************
*
*  calculate the 6 Rx numbers from Pfile header (for 11.x)
*  readrx01   050830  KL  Created readrx.c
*/
#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <math.h>
#include <string.h>
#include "toolsdata.h"


#if RECON_FLAG==1  /*KL*/
    #include "imagedb11x.h"		/* Signa header information */
    #include "rdbm11x.h"  
#endif
#if RECON_FLAG==2
    #include "imagedb12x.h"		/* Signa header information */
    #include "rdbm12x.h"  
#endif


/* Function declarations */
void instructions(), parsearg();
void readPFileHeader( POOL_HEADER *PFileHeader, FILE *PFile );
void readDataAcqTable( RDB_DATA_ACQ_TAB *dataAcqTab, FILE *pFile);
void readExamHeader( EXAMDATATYPE *examHDR, FILE *ImageFile);
void readSeriesHeader( SERIESDATATYPE *seriesHDR, FILE *ImageFile);
void readMRImageHeader( MRIMAGEDATATYPE *MRImageHDR, FILE *ImageFile);
int readrx(float *rx, char *pfile);
char *prefix;
int debug=0, briknote=0;
char *pfile;

main(int argc,char **argv)
{
   float rx[6];
   char hdr_notes[1000];
   parsearg(argv, argc);
   readrx(rx,pfile);
    if (briknote){   
sprintf(hdr_notes, "Start:\t%c%0.1f\t%c%0.1f\t%c%0.1f\nEnd:\t%c%0.1f\t%c%0.1f\t%c%0.1f\n", (rx[1]<0?'R':'L'), fabs(rx[1]),(rx[3]<0?'A':'P'),fabs(rx[3]),(rx[5]<0?'I':'S'),fabs(rx[5]),(rx[2]<0?'R':'L'), fabs(rx[2]),(rx[4]<0?'A':'P'),fabs(rx[4]),(rx[6]<0?'I':'S'),fabs(rx[6])); 
      
   if (strstr(prefix,".nii")!= NULL){
      sprintf(afni_notes,"nifti_tool -add_afni_ext 'Rx info:\n%s' -overwrite -infile %s",hdr_notes,prefix);
      system (afni_notes);
      printf("Rx info is written to %s as an AFNI extension (ecode=4).\n", prefix);
  }
  else{
      sprintf(afni_notes,"3dNotes -a 'Rx info:\n%s' %s+orig",hdr_notes,prefix);
      system (afni_notes);
      printf("Rx info is written to %s+orig.BRIK header.\n", prefix);
  }


}
}
void parsearg(argv, argc)
int argc;
char **argv;
{
        int i,nopt=0;
        if (debug) printf("argc=%d\n",argc);
        
        pfile=argv[argc-1];
        for (i=1; i<argc-1; i++) {
		if (!strncmp(argv[i], "-brik", 6)) {  /* dicome file*/
			if (++i >= argc) { instructions(argv); exit(0); }
			prefix = argv[i];
                        briknote =1;
			nopt++; nopt++;
		}
		else if (!strncmp(argv[i], "-debug", 6)) {
			debug = 1;
			nopt++;
                
                }
		else {
			printf("Option %s is not recognized\n",argv[i]);
                        instructions(argv);
			exit(0);
		}
	} 
       if (argc-nopt < 2){
		instructions(argv);
		exit(0);
	}    return;


}

void instructions(char **argv)
{
    printf("\nReading Rx info from GE Pfile header.");
    printf("\nUsage: %s [Options] pfile\n", argv[0]);
    printf("\nOptions: -debug        printing all information on stdout\n");
    printf("         -brik prefix  prefix of the dataset to whose header the Rx will be written.\n");
    printf("                       (Assumes NIFTI if prefix contains '.nii',  default AFNI briks!)\n");
    printf("This is version 1.0 (added NIFTI support) <kunlu@ucsd.edu>.\n");
    return;
}

int readrx(float *rx, char * pfile)
{
FILE *fin;
POOL_HEADER  PFileHeader;
SERIESDATATYPE *SeriesHDR;
MRIMAGEDATATYPE *MRImgHDR;
char startras, endras;
float startloc, endloc;
float ctr_R,ctr_A,ctr_S;
float norm_R,norm_A,norm_S;
float ctr_R2,ctr_A2,ctr_S2;
float thk, norm;

if( ( fin = fopen( pfile, "rb" ) ) == NULL ) {
     printf("Can't open pfile %s!\n",pfile);
}
else{
	printf("Retrieving Rx info from pfile %s!\n",pfile);
	readPFileHeader( &PFileHeader, fin );

	SeriesHDR=&(PFileHeader.rdb_hdr_series);
	MRImgHDR=&(PFileHeader.rdb_hdr_image);
	startras =SeriesHDR->start_ras;
	startloc =SeriesHDR->start_loc;
	endras =SeriesHDR->end_ras;
	endloc =SeriesHDR->end_loc;
	ctr_R=MRImgHDR->ctr_R;
	ctr_A=MRImgHDR->ctr_A;
	ctr_S=MRImgHDR->ctr_S;
	norm_R=MRImgHDR->norm_R;
	norm_A=MRImgHDR->norm_A;
	norm_S=MRImgHDR->norm_S;

	if ((startras =='A') || (startras =='P'))
		norm=norm_A;
	if ((startras =='S') || (startras =='I') )
		norm=norm_S;
	if ((startras =='R') || (startras =='L') )
		norm=norm_R;

	thk=endloc-startloc;
	ctr_R2=ctr_R+thk*norm_R/norm;
	ctr_A2=ctr_A+thk*norm_A/norm;
	ctr_S2=ctr_S+thk*norm_S/norm;
	rx[1]=-ctr_R;
	rx[2]=-ctr_R2;
	rx[3]=-ctr_A;
	rx[4]=-ctr_A2;
	rx[5]=ctr_S;
	rx[6]=ctr_S2;

	printf("norm_R= %5.4f, norm_A=%5.4f, norm_S=%5.4f\n",norm_R, norm_A,norm_S);

	printf("---------------------------------\n");
	printf(" \tR/L\tA/P\tS/I\n");
	printf("Start:\t%c%0.1f\t%c%0.1f\t%c%0.1f\n", (rx[1]<0?'R':'L'), fabs(rx[1]),(rx[3]<0?'A':'P'),fabs(rx[3]),(rx[5]<0?'I':'S'),fabs(rx[5])); 
	printf("End:\t%c%0.1f\t%c%0.1f\t%c%0.1f\n",(rx[2]<0?'R':'L'), fabs(rx[2]),(rx[4]<0?'A':'P'),fabs(rx[4]),(rx[6]<0?'I':'S'),fabs(rx[6])); 
	printf("---------------------------------\n");

	fclose(fin);
}
}

/******
*
* readPFileHeader( POOL_HEADER *, FILE *)
*
* This function reads the contents of the PFile header from
* the supplied file.
*
******/

void readPFileHeader( POOL_HEADER *PFileHeader, FILE *PFile ) {
    
    int offset;
    
  /*offset = 10240; 
    fseek(PFile, offset, SEEK_SET);
    readDataAcqTable(&(PFileHeader->rdb_hdr_data_acq_tab), PFile);

    offset = 57352; 
    fseek(PFile, offset, SEEK_SET);
    readExamHeader(&(PFileHeader->rdb_hdr_exam), PFile);
  */
    #if RECON_FLAG==1  /*KL*/
	offset = 58392; 
    #endif
    #if RECON_FLAG==2  /*KL*/
	offset = 62488; 
    #endif

    fseek(PFile, offset, SEEK_SET);
    readSeriesHeader(&(PFileHeader->rdb_hdr_series), PFile);

    #if RECON_FLAG==1  /*KL*/
	offset = 59928; 
    #endif
    #if RECON_FLAG==2  /*KL*/
	offset = 64536; 
    #endif

    fseek(PFile, offset, SEEK_SET);
    readMRImageHeader(&(PFileHeader->rdb_hdr_image), PFile);
    
    }

/******
*
* readDataAcqTable( RDB_DATA_ACQ_TAB *, FILE *)
*
* This function reads the contents of the data acquisition 
* table from the supplied file.
*
******/

void readDataAcqTable( RDB_DATA_ACQ_TAB *dataAcqTab, FILE *pFile) {
    
    if(fread(dataAcqTab,sizeof( RDB_DATA_ACQ_TAB ),1,pFile) != 1) { 
        printf( "Error in readDataAcqTable().\n" );
    }
}

/* end readDataAcqTable */


/******
*
* readExamHeader( EXAMDATATYPE *, FILE *)
*
* This function reads the contents of the exam data structure from
* the supplied file.
*
******/

void readExamHeader( EXAMDATATYPE *examHDR, FILE *ImageFile) {
    
    if(fread(examHDR,sizeof( EXAMDATATYPE ),1,ImageFile) != 1) { 
        printf( "\n. Error in readExamHeader()." );
    }
}

/* end readExamHeader */



/******
*
* readSeriesHeader( SERIESDATATYPE  *, FILE *)
*
* This function reads the contents of the series data structure from
* the supplied file.
*
******/

void readSeriesHeader( SERIESDATATYPE *seriesHDR, FILE *ImageFile) {

    if(fread(seriesHDR,sizeof( SERIESDATATYPE ),1,ImageFile) != 1) { 
        printf( "\n.Error in readSeriesHeader()." );    
    }

    
}

/* end readSeriesHeader */

/******
*
* readMRImageHeader( MRIMAGEDATATYPE *, FILE *)
*
* This function reads the contents of the MR image data structure from
* the supplied file.
*
******/

void readMRImageHeader( MRIMAGEDATATYPE *MRImageHDR, FILE *ImageFile) {
    
    if(fread(MRImageHDR,sizeof( MRIMAGEDATATYPE ),1,ImageFile) != 1) { 
        printf( "\n.Error in readMRImageHeader()." );   
    }
}

/* end readMRImageHeader */



