/* $Id: rhuser.h,v 1.2 2006/11/27 18:24:19 jperthen Exp $ */

/*******************************************************************************************
Definition of rhuser variable space.
Called by both gr and spep.
Number of bits per CV and rhuser structure are defined for rhusers storing multiple CVS. 
(NOTE: position (POSN) refers to the right end of the bit location) 

Date	       Author 	  Comments
06.09.05      JP	   rhuser usage for spep versions 38.0 and later.
06.11.21      JP	   rhusers pwpcasl, oppseq, sechNSte added for CBF quantification.

*******************************************************************************************

NOTES
 - table below lists the CVs assigned in spep.e (rhusers 0-23) and Prep.e (rhusers 24-48). 
 - rhusers are floats: 23 bits of significance available per rhuser. 
 - starred CVs are saved in the AFNI header by gr (version 17.0); accessible using '3dNotes'.
 - rhusers that remain unchanged to previous scheme: 1,2,5,6,7,10-13,15,31-33,36-39.
 - keep rhuser2=speprev; this is used by gr to set the recon pathway 
 - if making changes to BIT allocations, maintain backwards compatibility using 
   the following template:
 	#if (speprev*100)<3900
	#define cvBITS           x
	#else
	#define cvBITS           y
	#endif


  --------------------------------------------------------------------------------
  |  rhuser   		CV			Bits 			Notes
  --------------------------------------------------------------------------------
  Assigned in spep.e:
  --------------------------------------------------------------------------------
  |  0			  tmin*
  --------------------------------------------------------------------------------
  |  1			  reps*
  --------------------------------------------------------------------------------
  |  2  		  speprev*				    Do not change
  --------------------------------------------------------------------------------
  |  3			  opxres		  11
  |                        phases		  5
  |                        segments	          5
  --------------------------------------------------------------------------------
  |  4			  dda*
  --------------------------------------------------------------------------------
  |  5   		  gfov					   If gtype=0
  |                        opyres+skipline		       	   If gtype=1
  --------------------------------------------------------------------------------
  |  6			  gamp		   			   If gtype=0
  |			   skip	       				   If gtype=1
  --------------------------------------------------------------------------------
  |  7			  gslew		   			   If gtype=0
  |                        oscan	       			   If gtype=1
  --------------------------------------------------------------------------------
  |  8			  nl*			  6
  |  		  	   loopmode		  2
  |			   vte			  3         read into gr as bmapflag
  |			   sense		  4
  |			   sptraj		  4
  |			   fsat_pulse		  2
  --------------------------------------------------------------------------------
  |  9			  gtype			  4
  |			   sptype		            All the others are flags;
  | 			   spdir1*		  	      1 bit each
  |			   spdir2*
  |			   dif*
  |			   fsat*
  |		   	   fpmode
  |			   fiestamode
  |			   zbalance
  |			   rf1mode
  |			   oppseq*		2
  --------------------------------------------------------------------------------
  |  10			  opslspace
  --------------------------------------------------------------------------------
  |  11			  rsp_info[0].rsptloc + sloffset
  --------------------------------------------------------------------------------
  |  12			  rsp_info[1].rsptloc + sloffset
  --------------------------------------------------------------------------------
  |  13  		  tsp
  --------------------------------------------------------------------------------
  |  14			  bfactor
  --------------------------------------------------------------------------------
  |  15			  vte_dt
  --------------------------------------------------------------------------------
  |  16			  spmaxband
  --------------------------------------------------------------------------------
  |  17			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  18			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  19  		  		---EMPTY---
  --------------------------------------------------------------------------------
  |  20			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  21			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  22			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  23			 		---EMPTY---
  --------------------------------------------------------------------------------
  Assigned in Prep.e:
  --------------------------------------------------------------------------------
  |  24  		 qsatn		      	        3 
  |			   psatn		        3
  |    		           vsloop		        2
  |			   ntess			3
  |			   harp		        	3
  |			   hpoly		        3
  --------------------------------------------------------------------------------
  |  25			  tag*				4
  |			   q*				3     read into gr as quipss
  |			   nprep			3
  |			   casl 			2
  |		   	   bgsn*			3
  |			   tagaxis			2
  |			   vti				2
  --------------------------------------------------------------------------------
  |  26			  vcut
  --------------------------------------------------------------------------------
  |  27			  fixedtr			      All flags;1 bit each
  |			   psat_on*				
  |			   psat_after
  |			   bgs_on*
  |		   	   bgs_spatial*
  |			   bgspoil
  |			   postsat_on*
  |			   vsi_on
  |			   fair_slabsel*
  |			   fill_on
  --------------------------------------------------------------------------------
  |  28			  bgsti1
  --------------------------------------------------------------------------------
  |  29  		  bgsti2
  --------------------------------------------------------------------------------
  |  30			  bgsti3
  --------------------------------------------------------------------------------
  |  31			  pwafptot
  --------------------------------------------------------------------------------
  |  32  		  tagthick*
  --------------------------------------------------------------------------------
  |  33			  tagoffset*
  --------------------------------------------------------------------------------
  |  34			  tagsign
  --------------------------------------------------------------------------------
  |  35  		  tagslgap
  --------------------------------------------------------------------------------
  |  36  		  prepti1s
  --------------------------------------------------------------------------------
  |  37			  prepti1*
  --------------------------------------------------------------------------------
  |  38			  preptilast
  --------------------------------------------------------------------------------
  |  39  		  prepti*
  --------------------------------------------------------------------------------
  |  40			  bgsthick*
  --------------------------------------------------------------------------------
  |  41  		  vti_ramp	                4
  |                         vti_flat			4
  |                         nharp		        12
  --------------------------------------------------------------------------------
  |  42			  nsfairtagthick
  --------------------------------------------------------------------------------
  |  43  		  pwpcasl*
  --------------------------------------------------------------------------------
  |  44			  sechNSte*
  --------------------------------------------------------------------------------
  |  45  		  		---EMPTY---
  --------------------------------------------------------------------------------
  |  46			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  47			  		---EMPTY---
  --------------------------------------------------------------------------------
  |  48			 		---EMPTY---
  --------------------------------------------------------------------------------
*/


/* rhuser 3 */
#define opxresBITS       11
#define phasesBITS       5
#define segmentsBITS     5

#define opxresPOSN       0
#define phasesPOSN      (opxresPOSN + opxresBITS)
#define segmentsPOSN    (phasesPOSN + phasesBITS)


/* rhuser 8 */
#define nlBITS           6
#define loopmodeBITS     2
#define vteBITS          3
#define senseBITS        4
#define sptrajBITS       4
#define fsat_pulseBITS   2

#define nlPOSN	            0
#define loopmodePOSN   	   (nlPOSN + nlBITS)
#define vtePOSN 	   (loopmodePOSN + loopmodeBITS)
#define sensePOSN          (vtePOSN + vteBITS)
#define sptrajPOSN         (sensePOSN + senseBITS)
#define fsat_pulsePOSN     (sptrajPOSN + sptrajBITS)


/* rhuser 9: flags */

#define gtypeBITS           4
#define sptypeBITS          1
#define spdir1BITS          1
#define spdir2BITS          1
#define difBITS             1
#define fsatBITS            1
#define fpmodeBITS          1
#define fiestamodeBITS      1
#define zbalanceBITS        1
#define rf1modeBITS         1
#define oppseqBITS         2

#define gtypePOSN	    0
#define sptypePOSN	   (gtypePOSN + gtypeBITS)
#define spdir1POSN	   (sptypePOSN + sptypeBITS)
#define spdir2POSN	   (spdir1POSN+ spdir1BITS)
#define difPOSN		   (spdir2POSN+ spdir2BITS)
#define fsatPOSN	   (difPOSN + difBITS)
#define fpmodePOSN	   (fsatPOSN + fsatBITS)
#define fiestamodePOSN	   (fpmodePOSN + fpmodeBITS)
#define zbalancePOSN	   (fiestamodePOSN+ fiestamodeBITS)
#define rf1modePOSN	   (zbalancePOSN + zbalanceBITS)
#define oppseqPOSN	   (rf1modePOSN + rf1modeBITS)


/* rhuser 24 */
#define qsatnBITS	   3
#define psatnBITS	   3
#define vsloopBITS	   2
#define ntessBITS	   3
#define harpBITS	   3
#define hpolyBITS	   3

#define qsatnPOSN	  0
#define psatnPOSN        (qsatnPOSN + qsatnBITS)
#define vsloopPOSN       (psatnPOSN+ psatnBITS)
#define ntessPOSN	 (vsloopPOSN+ vsloopBITS)
#define harpPOSN	 (ntessPOSN+ ntessBITS)
#define hpolyPOSN	 (harpPOSN+ harpBITS)


/* rhuser 25 */
#define tagBITS		 4
#define qBITS		 3
#define nprepBITS	 3
#define caslBITS	 2
#define bgsnBITS	 3
#define tagaxisBITS	 2
#define vtiBITS		 2

#define tagPOSN		  0
#define qPOSN		 (tagPOSN + tagBITS)
#define nprepPOSN	 (qPOSN + qBITS)
#define caslPOSN         (nprepPOSN + nprepBITS)
#define bgsnPOSN	 (caslPOSN + caslBITS)
#define tagaxisPOSN	 (bgsnPOSN + bgsnBITS)
#define vtiPOSN          (tagaxisPOSN + tagaxisBITS)


/* rhuser 27 : flags */
#define fixedtrBITS	     1
#define psat_onBITS	     1
#define psat_afterBITS	     1
#define bgs_onBITS	     1
#define bgs_spatialBITS	     1
#define bgspoilBITS	     1
#define postsat_onBITS	     1
#define vsi_onBITS	     1
#define fair_slabselBITS     1
#define fill_onBITS	     1

#define fixedtrPOSN         0
#define psat_onPOSN        (fixedtrPOSN + fixedtrBITS)
#define psat_afterPOSN     (psat_onPOSN + psat_onBITS)
#define bgs_onPOSN         (psat_afterPOSN + psat_afterBITS)
#define bgs_spatialPOSN    (bgs_onPOSN + bgs_onBITS)
#define bgspoilPOSN        (bgs_spatialPOSN + bgs_spatialBITS)
#define postsat_onPOSN     (bgspoilPOSN + bgspoilBITS)
#define vsi_onPOSN         (postsat_onPOSN + postsat_onBITS)
#define fair_slabselPOSN   (vsi_onPOSN + vsi_onBITS)
#define fill_onPOSN        (fair_slabselPOSN + fair_slabselBITS)


/* rhuser 41 */
#define vti_rampBITS	  4
#define vti_flatBITS	  4
#define nharpBITS	  12

#define vti_rampPOSN     0
#define vti_flatPOSN    (vti_rampPOSN + vti_rampBITS)
#define  nharpPOSN      (vti_flatPOSN + vti_flatBITS)



